static char *sccsid = "@(#) ms07/public/oamlog.c, supportpacks, MS0C          1.1  01/01/10 10:33:47";
/*****************************************************************************/
/*                                                                           */
/* Module Name: oamlog.c                                                     */
/*                                                                           */
/* (C) IBM Corporation 2001                                                  */
/*                                                                           */
/* MS07 - Sample Authorisation Service                                       */
/*                                                                           */
/* PLEASE NOTE - This code is supplied "AS IS" with no                       */
/*              warranty or liability. It is not part of                     */
/*              any product.                                                 */
/*                                                                           */
/* Description: A sample Authorisation Service for MQSeries.                 */
/*                                                                           */
/*   This module implements the MQSeries Authorisation Service API, defined  */
/*   in the Programmable System Management book.  I hope it also explains    */
/*   some of what actually happens when the interface is invoked, and        */
/*   expands on the information in that book.                                */
/*                                                                           */
/*   It tracks requests made to a real authorisation component such as the   */
/*   OAM shipped with MQSeries, writing a log as it goes along.  It does not */
/*   make any authorisation checks itself.  A log may be useful to debug     */
/*   authorisation problems as you can see exactly what request (in          */
/*   particular, which userid) is being checked by the OAM.  Without the log */
/*   you might have to look at MQSeries trace files or worse ...             */
/*                                                                           */
/*   As it is supplied in source format, it could be used as the skeleton    */
/*   for a real authorisation service, to replace the IBM-supplied OAM.      */
/*                                                                           */
/*   See the readme supplied with this SupportPac for compilation,           */
/*   configuration and usage information.                                    */
/*                                                                           */
/* Author:                                                                   */
/*   Mark Taylor  (marke_taylor@uk.ibm.com)                                  */
/*   MQSeries Technical Strategy, IBM Hursley                                */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>      /* We won't use much in the way of system interfaces */
#include <string.h>

extern int getpid(void);

#include <cmqc.h>      /* MQSeries main header              */
#include <cmqzc.h>     /* MQSeries installable services hdr */


/*****************************************************************************/
/* This Authorisation module is intended to be run ONLY as the first in a    */
/* chain of installable services. As such, we want the return code given     */
/* to the queue manager to be one which forces the chain to continue.        */
/*                                                                           */
/* Some functions - Init, Term and the new Refresh - will continue the chain */
/* if a service component returns MQCC_OK but most do not. So I'll define    */
/* the return code which will force the chain to continue. If this module    */
/* were changed into one which did real authorisation checking, then I'd     */
/* change this value to MQCC_OK unless I really did want to return an error. */
/*                                                                           */
/* The only MQRC error values which are explicitly tested by the code that   */
/* invokes the Authorisation Service are                                     */
/*   MQRC_NOT_AUTHORIZED                                                     */
/*   MQRC_SERVICE_ERROR                                                      */
/*   MQRC_SERVICE_NOT_AVAILABLE                                              */
/*   MQRC_UNKNOWN_ENTITY                                                     */
/*   MQRC_UNKNOWN_OBJECT_NAME                                                */
/* A real auth service should normally set one of these. If the MQRC is not  */
/* one of these, then 'default' processing (which may vary by operation)     */
/* will be done. See also the comment about MQRC values in OACheckAuth.      */
/* Of course, if everything has worked OK and authorisation is granted,      */
/* then MQCC_OK/MQRC_NONE would be the correct response.                     */
/*****************************************************************************/
#define OA_DEF_CC (MQCC_WARNING)


/****************************************************************************/
/* This is where we'll write the logged information.                        */
/****************************************************************************/
#if MQAT_DEFAULT == MQAT_UNIX
#define LOGFILE "/var/mqm/errors/oamlog.log"
#else
#define LOGFILE "c:\\mqm\\errors\\oamlog.log"
#endif

/****************************************************************************/
/* A group  of process-wide global variables.                               */
/****************************************************************************/
FILE *fp = NULL;
char prefix[16];
int primary_process = 0;

/****************************************************************************/
/* Some utility functions/macros to dump information.                       */
/****************************************************************************/

/*
 * Print the Entity Type (user or group)
 */
#define OAETStr(x) \
  (x==MQZAET_PRINCIPAL?"User ":(x==MQZAET_GROUP?"Group":"Invalid Entity Type"))

/*
 * Decode the Options fields for Init/Term
 */
#define OATermOptStr(x) \
  ((x==MQZTO_SECONDARY)?"Secondary":"Primary")
#define OAInitOptStr(x) \
  ((x==MQZIO_SECONDARY)?"Secondary":"Primary")

/*
 * Print the Entity Name - for NT we will want to
 * also dump the domain. (The SID might be useful, but it's a
 * slightly more complex format to dump so I've left it out!)
 * I've also put the formatting string in the macro definition so it can
 * be used in the 'real' format string for printing the username.
 *
 * It seems that sometimes the entity name includes trailing spaces
 * and sometimes it doesn't. But at least it is always NULL-terminated.
 */
#if MQAT_DEFAULT == MQAT_UNIX
#define OAES "\"%s\""
#define OAEntityStr(x) ((x)->EntityNamePtr)
#else
#define OAES "\"%s@%s\""
#define OAEntityStr(x) \
 ((x)->EntityNamePtr), \
 ((x)->EntityDomainPtr)?(x)->EntityDomainPtr:"No Domain"
#endif

/*
 * Print the Object Type
 */
static char *OAOTStr(MQLONG x)
{
  /*
   * There are other defined MQOT_* values, but they should never get
   * sent to the OAM.
   */
  switch (x)
  {
  case MQOT_Q:        return "Queue";
  case MQOT_NAMELIST: return "NameList";
  case MQOT_PROCESS:  return "Process";
  case MQOT_Q_MGR:    return "QMgr";

  case MQOT_ALIAS_Q:  return "Alias Queue";
  case MQOT_MODEL_Q:  return "Model Queue";
  case MQOT_LOCAL_Q:  return "Local Queue";
  case MQOT_REMOTE_Q: return "Remote Queue";
  default:            return "Invalid Object Type";
  }
}

/*
 * Format the requested authorities. This function needs to put stuff into
 * a 'static' buffer so I've set it up to be passed the buffer from the
 * caller. That way we don't need to worry about multi-threaded calling
 * of the module. The buffer needs to be big enough to hold a complete
 * set of permissions; there is no error checking.
 *
 * This may not be the most efficient way of doing it, but I'm more
 * interested in readability than performance.
 *
 * It'll return a string looking something like
 *   0x00000024 [get set ]
 *
 * There are other defined MQZAO_* values, but they should never get
 * sent to the OAM. The setmqaut command knows about the 'allmqi'
 * group of operations, and to save space I'll print that instead of
 * the individual operations if all bits are set. I can't do
 * the same for the 'alladm' or 'all' sets because the definitions
 * include bits that are never (currently) used or set.
 *
 */
static char *OAAuthStr(MQLONG x,char *buf)
{
  sprintf(buf,"0x%08X [",x);

  if ((x & MQZAO_ALL_MQI)== MQZAO_ALL_MQI)
    strcat(buf,"allmqi ");
  else
  {
    if (x & MQZAO_CONNECT)               strcat(buf,"connect ");
    if (x & MQZAO_BROWSE )               strcat(buf,"browse ");
    if (x & MQZAO_INPUT  )               strcat(buf,"get ");
    if (x & MQZAO_OUTPUT )               strcat(buf,"put ");
    if (x & MQZAO_INQUIRE)               strcat(buf,"inq ");
    if (x & MQZAO_SET    )               strcat(buf,"set ");
    if (x & MQZAO_PASS_IDENTITY_CONTEXT) strcat(buf,"passid ");
    if (x & MQZAO_PASS_ALL_CONTEXT)      strcat(buf,"passall ");
    if (x & MQZAO_SET_IDENTITY_CONTEXT)  strcat(buf,"setid ");
    if (x & MQZAO_SET_ALL_CONTEXT)       strcat(buf,"setall ");
    if (x & MQZAO_ALTERNATE_USER_AUTHORITY) strcat(buf,"altusr ");
  }

  if (x & MQZAO_CREATE)                strcat(buf,"crt ");
  if (x & MQZAO_DELETE)                strcat(buf,"dlt ");
  if (x & MQZAO_DISPLAY)               strcat(buf,"dsp ");
  if (x & MQZAO_CHANGE)                strcat(buf,"chg ");
  if (x & MQZAO_CLEAR)                 strcat(buf,"clr ");

  strcat(buf,"]");
  return buf;
}

/****************************************************************************/
/* Now we get to the real functions which are registered to the qmgr.       */
/****************************************************************************/

/****************************************************************************/
/* Function: OATerm                                                         */
/*                                                                          */
/* Description:                                                             */
/*   The function is invoked during various termination stages (look at     */
/*   the Options field for PRIMARY/SECONDARY flags).                        */
/*                                                                          */
/*   SECONDARY termination is called in current qmgrs only when the         */
/*   process - amqzlaa0 or fastbound application - is about to end. There   */
/*   is no relationship between init/term and MQCONN/MQDISC operations as   */
/*   agents (amqzlaa0 processes) might have multiple threads. We don't get  */
/*   to see threads ending here.                                            */
/*                                                                          */
/*   PRIMARY termination is called once and only once when                  */
/*   the queue manager ends (and comes after all the SECONDARY terminations */
/*   - it's the reverse of init), and hence we're not going to close the    */
/*   logfile in this process until the final termination.                   */
/****************************************************************************/
static void MQENTRY OATerm(
  MQHCONFIG  hc,
  MQLONG     Options,
  PMQCHAR    pQMgrName,
  PMQBYTE    pComponentData,
  PMQLONG    pCompCode,
  PMQLONG    pReason)
{
  /*
   * I'm not going to make use of my knowledge that PRIMARY==0
   * and that PRIMARY/SECONDARY are the only options currently defined
   * to shorten this next line!
   */
  if ((primary_process) && ((Options & MQZTO_PRIMARY) == MQZTO_PRIMARY) ||
	                   ((Options & MQZTO_SECONDARY) == MQZTO_SECONDARY))
  {
    if (fp)  /* Let's be sure! */
    {
      fprintf(fp,"[%s] OATerm\n\tQMgr  : \"%48.48s\"\n\tOpts  : 0x%08X [%s]\n",
	    prefix,pQMgrName,Options,OATermOptStr(Options));
      fclose(fp);
      fp = NULL;
    }
  }
  *pCompCode = MQCC_OK;
  *pReason   = MQRC_NONE;
  return;
}

/****************************************************************************/
/* Function: OADeleteAuth                                                   */
/*                                                                          */
/* Description:                                                             */
/*   Remove the authority for all users of an object - the actual object    */
/*   is about to be deleted. It's possible that a tempdyn queue may already */
/*   have been deleted, and this is a just-to-make-sure thing done during   */
/*   restart so a real OAM should perhaps return 'OK' even if the ACLs      */
/*   are no longer in its configuration.                                    */
/****************************************************************************/
static void MQENTRY OADeleteAuth(
  PMQCHAR  pQMgrName,
  PMQCHAR  pObjectName,
  MQLONG   ObjectType,
  PMQBYTE  pComponentData,
  PMQLONG  pContinuation,
	
  PMQLONG  pCompCode,
  PMQLONG  pReason)
{
  fprintf(fp,"[%s] OADeleteAuth\n\tObject: \"%48.48s\" [%s]\n",
	  prefix,
	  pObjectName,
	  OAOTStr(ObjectType));
  *pCompCode = OA_DEF_CC;
  *pReason   = MQRC_NONE;
  *pContinuation = MQZCI_CONTINUE;
  return;
}

/****************************************************************************/
/* Function: OARefreshCache                                                 */
/*                                                                          */
/* Description:                                                             */
/*   Only available with MQSeries V5.2, this tells the OAM to refresh       */
/*   any cache of userid/group/authorisation information that it might hold.*/
/*                                                                          */
/* Note:                                                                    */
/*   This function CAN return MQCC_OK as the queue manager invokes all      */
/*   service components in the chain, whatever the return code.             */
/****************************************************************************/
static void MQENTRY OARefreshCache(
  PMQCHAR  pQMgrName,
  PMQBYTE  pComponentData,
  PMQLONG  pContinuation,
  PMQLONG  pCompCode,
  PMQLONG  pReason)
{
  fprintf(fp,"[%s] OARefreshCache\n", prefix);
  *pCompCode = MQCC_OK;
  *pReason   = MQRC_NONE;
  *pContinuation = MQZCI_CONTINUE;
  return;
}

/****************************************************************************/
/* Function: OAGetAuth                                                      */
/*                                                                          */
/* Description:                                                             */
/*   Called to return the current Authority assigned to this user/group.    */
/*   Because the Authority is a value returned from the OAM later in the    */
/*   chain, we can't access it here.                                        */
/*                                                                          */
/*   The value returned from the IBM-shipped OAM would be the union of the  */
/*   authorities assigned to all groups to which the user belongs.          */
/****************************************************************************/
static void MQENTRY OAGetAuth(
  PMQCHAR  pQMgrName,
  PMQZED   pEntityData,
  MQLONG   EntityType,
  PMQCHAR  pObjectName,
  MQLONG   ObjectType,
  PMQLONG  pAuthority,
  PMQBYTE  pComponentData,
  PMQLONG  pContinuation,
  PMQLONG  pCompCode,
  PMQLONG  pReason)
{
  fprintf(fp,"[%s] OAGetAuth\n\tObject: \"%48.48s\" [%s]\n\t%-6.6s: " OAES "\n",
	  prefix,
	  pObjectName,
	  OAOTStr(ObjectType),
	  OAETStr(EntityType),
	  OAEntityStr(pEntityData));
  *pCompCode = OA_DEF_CC;
  *pReason   = MQRC_NONE;
  *pContinuation = MQZCI_CONTINUE;
  return;
}

/****************************************************************************/
/* Function: OAGetExplicitAuth                                              */
/*                                                                          */
/* Description:                                                             */
/*   This is called to return the Authority assigned to a particular user   */
/*   or group. What's the difference between Get and GetExplicit?           */
/*   The answer (in the IBM-shipped OAM) is whether or not secondary groups */
/*   are considered. This is used during creation of new object, where we   */
/*   want to find the authority of the creating user. If you're writing     */
/*   your own OAM, then you might want to make the two Get functions do the */
/*   same thing.                                                            */
/*                                                                          */
/* Note:                                                                    */
/*   The header file cmqzc.h suggests this has an additional parameter,     */
/*   called AuthorityMask, but it doesn't! The header file is wrong, and    */
/*   will be corrected.                                                     */
/****************************************************************************/
static void MQENTRY OAGetExplicitAuth(
  PMQCHAR  pQMgrName,
  PMQZED   pEntityData,
  MQLONG   EntityType,
  PMQCHAR  pObjectName,
  MQLONG   ObjectType,
  PMQLONG  pAuthority,
  PMQBYTE  pComponentData,
  PMQLONG  pContinuation,
  PMQLONG  pCompCode,
  PMQLONG  pReason)
{
  fprintf(fp,"[%s] OAGetExplicitAuth\n\tObject: \"%48.48s\" [%s]\n\t%-6.6s: " OAES "\n",
	  prefix,
	  pObjectName,
	  OAOTStr(ObjectType),
	  OAETStr(EntityType),
	  OAEntityStr(pEntityData));
  *pCompCode = OA_DEF_CC;
  *pReason   = MQRC_NONE;
  *pContinuation = MQZCI_CONTINUE;
  return;
}

/****************************************************************************/
/* Function: OASetAuth                                                      */
/*                                                                          */
/* Description:                                                             */
/*   Sets the authority for a user to the object. Called whenever you run   */
/*   setmqaut and also when objects are created.                            */
/****************************************************************************/
static void MQENTRY OASetAuth(
  PMQCHAR  pQMgrName,
  PMQZED   pEntityData,
  MQLONG   EntityType,
  PMQCHAR  pObjectName,
  MQLONG   ObjectType,
  MQLONG   Authority,
  PMQBYTE  pComponentData,
  PMQLONG  pContinuation,
  PMQLONG  pCompCode,
  PMQLONG  pReason)
{
  char buf[128] = {0};
  fprintf(fp,"[%s] OASetAuth\n\tObject: \"%48.48s\" [%s]\n\t%-6.6s: " OAES "\n\tAuth  : %s\n",
	  prefix,
	  pObjectName,
	  OAOTStr(ObjectType),
	  OAETStr(EntityType),
	  OAEntityStr(pEntityData),
	  OAAuthStr(Authority,buf));
  *pCompCode = OA_DEF_CC;
  *pReason   = MQRC_NONE;
  *pContinuation = MQZCI_CONTINUE;
  return;
}

/****************************************************************************/
/* Function: OACopyAllAuth                                                  */
/*                                                                          */
/* Description:                                                             */
/*   Sets the authority for a new object, based on a reference set of       */
/*   permissions. Obviously both the reference and the new object have      */
/*   the same object type. In fact, it's only called (today) during         */
/*   creation of a dynamic queue. Unfortunately there's nothing in this     */
/*   interface which says whether or not it's a temporary or permanent      */
/*   dynamic queue, which could be useful to know, as TDQs expire along     */
/*   with the qmgr and info about them need not be hardened.                */
/****************************************************************************/
static void MQENTRY OACopyAllAuth(
  PMQCHAR  pQMgrName,
  PMQCHAR  pRefObjectName,
  PMQCHAR  pObjectName,
  MQLONG   ObjectType,
  PMQBYTE  pComponentData,
  PMQLONG  pContinuation,
  PMQLONG  pCompCode,
  PMQLONG  pReason)
{
  fprintf(fp,"[%s] OACopyAllAuth\n\tFrom  : \"%48.48s\" [%s]\n\tTo    : \"%48.48s\"\n",
	  prefix,
	  pRefObjectName,
	  OAOTStr(ObjectType),
	  pObjectName);
  *pCompCode = OA_DEF_CC;
  *pReason   = MQRC_NONE;
  *pContinuation = MQZCI_CONTINUE;
  return;
}

/****************************************************************************/
/* Function: OACheckAuth                                                    */
/*                                                                          */
/* Description:                                                             */
/*   Called whenever the qmgr wants to see if someone can do something.     */
/*   Typically this will be during MQCONN or MQOPEN. Remember, we're never  */
/*   called during MQPUT/MQGET.                                             */
/*                                                                          */
/*   The qmgr code will break out of the chaining loop whatever the CC      */
/*   and whatever the Continuation parameters say,                          */
/*   unless some very specific error codes happen. This is so that          */
/*   one, and only one, service returns a response granting access to the   */
/*   object. But for this particular service we want the chain always to    */
/*   continue. So I set a combination of Reason/CC/Continuation which will  */
/*   pass the qmgr checks and then move on to the real OAM.                 */
/*                                                                          */
/*   A proper authorisation component would do some real work here! And     */
/*   then set MQCC_OK (or FAILED) and some appropriate error code.          */
/****************************************************************************/
static void MQENTRY OACheckAuth (
  PMQCHAR  pQMgrName,
  PMQZED   pEntityData,
  MQLONG   EntityType,
  PMQCHAR  pObjectName,
  MQLONG   ObjectType,
  MQLONG   Authority,
  PMQBYTE  pComponentData,
  PMQLONG  pContinuation,
  PMQLONG  pCompCode,
  PMQLONG  pReason)
{
  char buf[128] = {0};
  fprintf(fp,"[%s] OACheckAuth\n\tObject: \"%48.48s\" [%s]\n\t%-6.6s: " OAES "\n\tAuth  : %s\n",
	  prefix,
	  pObjectName,
	  OAOTStr(ObjectType),
	  OAETStr(EntityType),
	  OAEntityStr(pEntityData),
	  OAAuthStr(Authority,buf));

  *pCompCode = OA_DEF_CC;
  *pReason   = MQRC_UNKNOWN_OBJECT_NAME;
  *pContinuation = MQZCI_CONTINUE;
  return;
}

/****************************************************************************/
/* Function: MQStart                                                        */
/*                                                                          */
/* Description:                                                             */
/*   This is the initialisation and entrypoint for the dynamically loaded   */
/*   authorisation installable service. It registers the addresses of the   */
/*   other functions which are to be called by the qmgr.                    */
/*                                                                          */
/*   This function is called whenever the module is loaded.  The Options    */
/*   field will show whether it's a PRIMARY (ie during qmgr startup) or     */
/*   SECONDARY (any other time - normally during the start of an agent      */
/*   process which is not necessarily the same as during MQCONN, especially */
/*   when running multi-threaded agents) initialisation, but there's        */
/*   nothing different that we'd want to do here based on that flag.        */
/*                                                                          */
/*   Because of when the init function is called, there is no need to       */
/*   worry about multi-threaded stuff in this particular function, and      */
/*   hence no need to do something like a lock around the fopen() call.     */
/*                                                                          */
/*   We do not use ComponentData at all in this module and the qm.ini       */
/*   attribute which references it should be set to zero. But if it were    */
/*   non-zero, then a block of data is allocated from a chunk of            */
/*   shared memory and available for all processes running this module      */
/*   within this queue manager. It could be used to hold truly global       */
/*   state info, but then you'd need to manage potential simultaneous       */
/*   updates to it.                                                         */
/*                                                                          */
/* Note:                                                                    */
/*   This function MUST be called MQStart on some platforms, so we'll use   */
/*   the same name for all of them.                                         */
/****************************************************************************/
void MQENTRY MQStart(
  MQHCONFIG hc,
  MQLONG    Options,
  MQCHAR48  QMgrName,
  MQLONG    ComponentDataLength,
  PMQBYTE   ComponentData,
  PMQLONG   Version,
  PMQLONG   pCompCode,
  PMQLONG   pReason)
{
  MQLONG CC       = MQCC_OK;
  MQLONG Reason   = MQRC_NONE;

  /**************************************************************************/
  /* Only 1 process (amqzxma0) gets this per qmgr but that process may      */
  /* still also get a 2ary init later so we don't have an 'else' clause -   */
  /* the default has already been set at the top of the file.               */
  /**************************************************************************/
  if ((Options & MQZIO_PRIMARY) == MQZIO_PRIMARY)
    primary_process = 1;

  if (!fp)
  {
    fp = fopen(LOGFILE,"a");
    sprintf(prefix,"%d",getpid());
  }

  if (fp)
  {
    setbuf(fp,NULL);    /* try to reduce interleaved output; get auto-flush */

    /************************************************************************/
    /* Initialise the entry point vectors.  This is performed for both      */
    /* global and process initialisation, ie whatever the value of the      */
    /* Options field.                                                       */
    /************************************************************************/
    if (CC == MQCC_OK)
      MQZEP(hc, MQZID_INIT_AUTHORITY,(PMQFUNC)MQStart,&CC,&Reason);

    if (CC == MQCC_OK)
      MQZEP(hc,MQZID_TERM_AUTHORITY,(PMQFUNC)OATerm,&CC,&Reason);

    if (CC == MQCC_OK)
      MQZEP(hc,MQZID_CHECK_AUTHORITY,(PMQFUNC)OACheckAuth,&CC,&Reason);

    if (CC == MQCC_OK)
      MQZEP(hc,MQZID_COPY_ALL_AUTHORITY,(PMQFUNC)OACopyAllAuth,&CC,&Reason);

    if (CC == MQCC_OK)
      MQZEP(hc,MQZID_DELETE_AUTHORITY,(PMQFUNC)OADeleteAuth,&CC,&Reason);

    if (CC == MQCC_OK)
      MQZEP(hc,MQZID_SET_AUTHORITY,(PMQFUNC)OASetAuth,&CC,&Reason);

    if (CC == MQCC_OK)
      MQZEP(hc,MQZID_GET_AUTHORITY,(PMQFUNC)OAGetAuth,&CC,&Reason);

    if (CC == MQCC_OK)
      MQZEP(hc,MQZID_GET_EXPLICIT_AUTHORITY,(PMQFUNC)OAGetExplicitAuth,&CC,&Reason);

#ifdef MQZID_REFRESH_CACHE
    /************************************************************************/
    /* This is only defined on V5.2 systems. Don't try to compile this code */
    /* on a 5.2 machine and then run it on 5.1.                             */
    /************************************************************************/
    if (CC == MQCC_OK)
      MQZEP(hc,MQZID_REFRESH_CACHE,(PMQFUNC)OARefreshCache,&CC,&Reason);
#endif

    fprintf(fp,"[%s] OAInit\n\tQMgr  : \"%48.48s\"\n\tCC: %d RC: %d Options: 0x%08X [%s]\n",
	    prefix,QMgrName,CC,Reason,Options,OAInitOptStr(Options));
  }

  if ((!fp) || (CC != MQCC_OK))
  {
    CC       = MQCC_FAILED;
    Reason   = MQRC_INITIALIZATION_FAILED;
  }


  /**************************************************************************/
  /* There have been several versions of the Auth Service interface.        */
  /* We will tell the qmgr to call us with the V2 parameters, which are     */
  /* needed on Windows NT to get the SID and domain name information.       */
  /* If MQZID_REFRESH_CACHE value is defined, then we can do the V3 version */
  /**************************************************************************/
#ifdef MQZID_REFRESH_CACHE
  *Version   = MQZAS_VERSION_3;
#else
  *Version   = MQZAS_VERSION_2;
#endif

  /**************************************************************************/
  /* And set the return codes.                                              */
  /**************************************************************************/
  *pCompCode = CC;
  *pReason   = Reason;

  return;
}
